@@ -1,8 +1,8 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2015 Andrew Ziem
-# http://bleachbit.sourceforge.net
+# Copyright (C) 2008-2016 Andrew Ziem
+# http://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -42,11 +42,12 @@ elif 'nt' == os.name:
     import Windows
 
 from Common import _
-from FileUtilities import children_in_directory
+from FileUtilities import children_in_directory, expandvars
 from Options import options
 
+# Suppress GTK warning messages while running in CLI #34
 warnings.simplefilter("ignore", Warning)
-                      # Supress GTK warning messages while running in CLI #34
+
 try:
     import gtk
     HAVE_GTK = True
@@ -134,7 +135,7 @@ class Cleaner:
         """Yield the names and descriptions of each option in a 2-tuple"""
         if self.options:
             for key in sorted(self.options.keys()):
-                yield ((self.options[key][0], self.options[key][1]))
+                yield (self.options[key][0], self.options[key][1])
 
     def get_options(self):
         """Return user-configurable options in 2-tuple (id, name)"""
@@ -163,7 +164,7 @@ class Cleaner:
                     print "debug: process '%s' is running" % pathname
                     return True
             elif 'pathname' == test:
-                expanded = os.path.expanduser(os.path.expandvars(pathname))
+                expanded = os.path.expanduser(expandvars(pathname))
                 for globbed in glob.iglob(expanded):
                     if os.path.exists(globbed):
                         print "debug: file '%s' exists indicating '%s' is running" % (globbed, self.name)
@@ -208,6 +209,8 @@ class Firefox(Cleaner):
             'Site preferences'), _('Settings for individual sites'))
         self.add_option('passwords', _('Passwords'), _(
             'A database of usernames and passwords as well as a list of sites that should not store passwords'))
+        self.set_warning(
+            'passwords', _('This option will delete your saved passwords.'))
         self.add_option(
             'url_history', _('URL history'), _('List of visited web pages'))
         self.add_option('vacuum', _('Vacuum'), _(
@@ -267,7 +270,7 @@ class Firefox(Cleaner):
         if 'posix' == os.name:
             crashdir = os.path.expanduser("~/.mozilla/firefox/Crash Reports")
         if 'nt' == os.name:
-            crashdir = os.path.expandvars(
+            crashdir = expandvars(
                 "$USERPROFILE\\Application Data\\Mozilla\\Firefox\\Crash Reports")
         if 'crash_reports' == option_id:
             for filename in children_in_directory(crashdir, False):
@@ -295,12 +298,15 @@ class Firefox(Cleaner):
                 self.profile_dir, "formhistory.sqlite")
         # passwords
         if 'passwords' == option_id:
+            # http://kb.mozillazine.org/Password_Manager
             files += FileUtilities.expand_glob_join(
                 self.profile_dir, "signons.txt")
             files += FileUtilities.expand_glob_join(
                 self.profile_dir, "signons[2-3].txt")
             files += FileUtilities.expand_glob_join(
                 self.profile_dir, "signons.sqlite")
+            files += FileUtilities.expand_glob_join(
+                self.profile_dir, "logins.json")
         # session restore
         if 'session_restore' == option_id:
             # Names include sessionstore.js, sessionstore.bak,
@@ -439,7 +445,7 @@ class System(Cleaner):
         # options for Linux and BSD
         #
         if 'posix' == os.name:
-            # TRANSLATORS: desktop entries are .desktop files in Linux tha
+            # TRANSLATORS: desktop entries are .desktop files in Linux that
             # make up the application menu (the menu that shows BleachBit,
             # Firefox, and others.  The .desktop files also associate file
             # types, so clicking on an .html file in Nautilus brings up
@@ -614,7 +620,7 @@ class System(Cleaner):
                 '$windir\\system32\\wbem\\Logs\\*.log', )
 
             for path in paths:
-                expanded = os.path.expandvars(path)
+                expanded = expandvars(path)
                 for globbed in glob.iglob(expanded):
                     files += [globbed]
 
@@ -626,10 +632,10 @@ class System(Cleaner):
         # how to manually create this file
         # http://www.pctools.com/guides/registry/detail/856/
         if 'nt' == os.name and 'memory_dump' == option_id:
-            fname = os.path.expandvars('$windir\\memory.dmp')
+            fname = expandvars('$windir\\memory.dmp')
             if os.path.exists(fname):
                 files += [fname]
-            for fname in glob.iglob(os.path.expandvars('$windir\\Minidump\\*.dmp')):
+            for fname in glob.iglob(expandvars('$windir\\Minidump\\*.dmp')):
                 files += [fname]
 
         # most recently used documents list
@@ -670,7 +676,7 @@ class System(Cleaner):
 
         # temporary files
         if 'nt' == os.name and 'tmp' == option_id:
-            dirname = os.path.expandvars(
+            dirname = expandvars(
                 "$USERPROFILE\\Local Settings\\Temp\\")
             # whitelist the folder %TEMP%\Low but not its contents
             # https://bugs.launchpad.net/bleachbit/+bug/1421726
@@ -678,7 +684,7 @@ class System(Cleaner):
             for filename in children_in_directory(dirname, True):
                 if not low == filename.lower():
                     yield Command.Delete(filename)
-            dirname = os.path.expandvars("$windir\\temp\\")
+            dirname = expandvars("$windir\\temp\\")
             for filename in children_in_directory(dirname, True):
                 yield Command.Delete(filename)
 
@@ -741,7 +747,7 @@ class System(Cleaner):
 
         # prefetch
         if 'nt' == os.name and 'prefetch' == option_id:
-            for path in glob.iglob(os.path.expandvars('$windir\\Prefetch\\*.pf')):
+            for path in glob.iglob(expandvars('$windir\\Prefetch\\*.pf')):
                 yield Command.Delete(path)
 
         # recycle bin
@@ -797,8 +803,13 @@ class System(Cleaner):
         regexes.append('^' + os.path.expanduser('~/.cache/wallpaper/'))
         # Clean Firefox cache from Firefox cleaner (LP#1295826)
         regexes.append('^' + os.path.expanduser('~/.cache/mozilla'))
+        # Clean Google Chrome cache from Google Chrome cleaner (LP#656104)
+        regexes.append('^' + os.path.expanduser('~/.cache/google-chrome'))
         regexes.append(
             '^' + os.path.expanduser('~/.cache/gnome-control-center/'))
+        # iBus Pinyin
+        # https://bugs.launchpad.net/bleachbit/+bug/1538919
+        regexes.append('^' + os.path.expanduser('~/.cache/ibus/'))
         for regex in regexes:
             if None != re.match(regex, pathname):
                 return True
@@ -844,6 +855,8 @@ def create_simple_cleaner(paths):
                 if not isinstance(path, (str, unicode)):
                     raise RuntimeError(
                         'expected path as string but got %s' % str(path))
+                if not os.path.isabs(path):
+                    path = os.path.abspath(path)
                 if os.path.isdir(path):
                     for child in children_in_directory(path, True):
                         yield Command.Shred(child)
