@@ -2,8 +2,8 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2015 Andrew Ziem
-# http://bleachbit.sourceforge.net
+# Copyright (C) 2008-2016 Andrew Ziem
+# http://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -92,31 +92,39 @@ def __random_string(length):
 
 
 def bytes_to_human(bytes_i):
-    """Display a file size in human terms (megabytes, etc.) using SI standard"""
+    """Display a file size in human terms (megabytes, etc.) using preferred standard (SI or IEC)"""
 
-    storage_multipliers = {1000 ** 5: 'PB', 1000 ** 4: 'TB',
-                           1000 ** 3: 'GB', 1000 ** 2: 'MB', 1000: 'kB', 1: 'B'}
+    if bytes_i < 0:
+        return '-' + bytes_to_human(-bytes_i)
+
+    from Options import options
+    if options.get('units_iec'):
+        prefixes = ['', 'Ki', 'Mi', 'Gi', 'Ti', 'Pi']
+        base = 1024.0
+    else:
+        prefixes = ['', 'k', 'M', 'G', 'T', 'P']
+        base = 1000.0
 
     assert(isinstance(bytes_i, (int, long)))
 
     if 0 == bytes_i:
         return "0"
 
-    if bytes_i >= 1000 ** 3:
+    if bytes_i >= base ** 3:
         decimals = 2
-    elif bytes_i >= 1000:
+    elif bytes_i >= base:
         decimals = 1
     else:
         decimals = 0
 
-    for key in sorted(storage_multipliers.keys(), reverse=True):
-        if bytes_i >= key:
-            abbrev = round((1.0 * bytes_i) / key, decimals)
-            suf = storage_multipliers[key]
-            return locale.str(abbrev) + suf
-
-    if bytes_i < 0:
-        return "-" + bytes_to_human(abs(bytes_i))
+    for exponent in range(0, len(prefixes)):
+        if bytes_i < base:
+            abbrev = round(bytes_i, decimals)
+            suf = prefixes[exponent]
+            return locale.str(abbrev) + suf + 'B'
+        else:
+            bytes_i /= base
+    return 'A lot.'
 
 
 def children_in_directory(top, list_directories=False):
@@ -233,7 +241,7 @@ def delete(path, shred=False, ignore_missing=False, allow_shred=True):
             # [Errno 39] Directory not empty
             # https://bugs.launchpad.net/bleachbit/+bug/1012930
             if errno.ENOTEMPTY == e.errno:
-                logger.info("directory is not empty: %s" % path)
+                logger.info("directory is not empty: %s", path)
             else:
                 raise
         except WindowsError, e:
@@ -242,7 +250,7 @@ def delete(path, shred=False, ignore_missing=False, allow_shred=True):
             # Error 145 may happen if the files are scheduled for deletion
             # during reboot.
             if 145 == e.winerror:
-                logger.info("directory is not empty: %s" % path)
+                logger.info("directory is not empty: %s", path)
             else:
                 raise
     elif os.path.isfile(path):
@@ -252,14 +260,14 @@ def delete(path, shred=False, ignore_missing=False, allow_shred=True):
                 wipe_contents(path)
             except IOError, e:
                 # permission denied (13) happens shredding MSIE 8 on Windows 7
-                logger.debug(" IOError #%s shredding '%s'" % (e.errno, path))
+                logger.debug("IOError #%s shredding '%s'", e.errno, path)
             # wipe name
             os.remove(wipe_name(path))
         else:
             # unlink
             os.remove(path)
     else:
-        logger.info("special file type cannot be deleted: %s" % path)
+        logger.info("special file type cannot be deleted: %s", path)
 
 
 def ego_owner(filename):
@@ -315,12 +323,24 @@ def expand_glob_join(pathname1, pathname2):
     """Join pathname1 and pathname1, expand pathname, glob, and return as list"""
     ret = []
     pathname3 = os.path.expanduser(
-        os.path.expandvars(os.path.join(pathname1, pathname2)))
+        expandvars(os.path.join(pathname1, pathname2)))
     for pathname4 in glob.iglob(pathname3):
         ret.append(pathname4)
     return ret
 
 
+def expandvars(path):
+    if (2, 5) == sys.version_info[0:2] and 'nt' == os.name:
+        # Python 2.5 should not change $foo when foo is unknown, but
+        # it actually strips it out.
+        import backport
+        expandvars = backport.expandvars
+        return backport.expandvars(path)
+    else:
+        expandvars = os.path.expandvars
+        return os.path.expandvars(path)
+
+
 def extended_path(path):
     """If applicable, return the extended Windows pathname"""
     if 'nt' == os.name:
@@ -335,13 +355,8 @@ def extended_path(path):
 def free_space(pathname):
     """Return free space in bytes"""
     if 'nt' == os.name:
-        import ctypes
-        free_bytes = ctypes.c_int64()
-        rc = ctypes.windll.kernel32.GetDiskFreeSpaceExW(unicode(pathname),
-                                                        ctypes.byref(free_bytes), None, None)
-        if 0 == rc:
-            raise ctypes.WinError()
-        return free_bytes.value
+        _, _, free_bytes = win32file.GetDiskFreeSpaceEx(pathname)
+        return free_bytes
     mystat = os.statvfs(pathname)
     return mystat.f_bfree * mystat.f_bsize
 
@@ -409,26 +424,20 @@ def guess_overwrite_paths():
         if not same_partition(home, '/tmp/'):
             ret.append('/tmp')
     elif 'nt' == os.name:
-        localtmp = os.path.expandvars('$TMP')
+        localtmp = expandvars('$TMP')
         logger = logging.getLogger(__name__)
         if not os.path.exists(localtmp):
-            logger.warning('%TMP% does not exist: %s' % localtmp)
+            logger.warning('%TMP% does not exist: %s', localtmp)
             localtmp = None
         else:
             from win32file import GetLongPathName
             localtmp = GetLongPathName(localtmp)
         from Windows import get_fixed_drives
         for drive in get_fixed_drives():
-            try:
-                if localtmp and same_partition(localtmp, drive):
-                    ret.append(localtmp)
-                else:
-                    ret.append(drive)
-            except Exception, e:
-                # see https://github.com/az0/bleachbit/issues/27
-                # https://bugs.launchpad.net/bleachbit/+bug/1372179
-                logger.error('error in same_partition(%s, %s): %s' %
-                             (localtmp, drive, Common.decode_str(e)))
+            if localtmp and same_partition(localtmp, drive):
+                ret.append(localtmp)
+            else:
+                ret.append(drive)
     else:
         NotImplementedError('Unsupported OS in guess_overwrite_paths')
     return ret
@@ -476,13 +485,14 @@ def same_partition(dir1, dir2):
     if 'nt' == os.name:
         try:
             return free_space(dir1) == free_space(dir2)
-        except IOError, e:
-            if e.errno == errno.EACCES:
-                logger = logging.getLogger(__name__)
-                logger.error(
-                    'access denied for same_partition(%s, %s): %s' % (dir1, dir2, Common.decode_str(e)))
+        except pywintypes.error, e:
+            if 5 == e.winerror:
+                # Microsoft Office 2010 Starter Edition has a virtual
+                # drive that gives access denied
+                # https://bugs.launchpad.net/bleachbit/+bug/1372179
+                # https://bugs.launchpad.net/bleachbit/+bug/1474848
                 # https://github.com/az0/bleachbit/issues/27
-                return False
+                return dir1[0] == dir2[0]
             raise
     stat1 = os.statvfs(dir1)
     stat2 = os.statvfs(dir2)
@@ -558,103 +568,6 @@ def wipe_contents(path, truncate=True):
     f.close()
 
 
-def wipe_inodes(pathname, idle=False):
-    """Create empty files with long names to clear remanence in metadata"""
-    logger = logging.getLogger(__name__)
-    # When the file system is full, Linux 3.5 will sometimes
-    # give an error and sometimes succeed.
-    errors = 0
-    files = []
-    if 'nt' == os.name:
-        # 255 according to http://en.wikipedia.org/wiki/Long_filename
-        f_namemax = 250
-        if not pathname.endswith('\\'):
-            pathname += '\\'
-    elif 'posix' == os.name:
-        f_namemax = os.statvfs(pathname).f_namemax
-        stats = os.statvfs(pathname)
-        inodes_avail_original = stats.f_favail
-    suffix = ' ' * (f_namemax - len(pathname) - 10)
-    logger.info('creating empty files')
-    logger.debug('debug: suffix length = %d' % len(suffix))
-    last_idle = time.time()
-    # Creating the files in a sub-directory improves recovery
-    # of disk space:
-    # http://bleachbit.sourceforge.net/news/deleting-files-doesnt-free-disk-space
-    tmpdir = os.path.join(pathname, __random_string(10))
-    os.mkdir(tmpdir)
-
-    # create empty files until file system is exhausted
-    while True:
-        try:
-            (fd, fn) = tempfile.mkstemp(dir=tmpdir, suffix=suffix, prefix='')
-        except KeyboardInterrupt:
-            logger.info('wipe_inodes: keyboard interrupt')
-            break
-        except Exception, e:
-            logger.error(Common.decode_str(e))
-            errors += 1
-            if os.path.exists(fn):
-                # on Windows with FAT32, files sometimes exist despite error
-                files.append(fn)
-            try:
-                os.close(fd)
-            except:
-                pass
-        else:
-            errors = 0
-            files.append(fn)
-            os.close(fd)
-            if idle and (time.time() - last_idle) > 2:
-                stats = os.statvfs(pathname)
-                percent_done = None  # not implemented on Windows
-                if 'posix' == os.name:
-                    if inodes_avail_original > 0:
-                        percent_done = 1.0 * \
-                            (inodes_avail_original - stats.f_favail) / \
-                            inodes_avail_original
-                    else:
-                        percent_done = 1.0
-                yield (2, percent_done, None)
-                last_idle = time.time()
-            # In case the application closes prematurely, make sure this file
-            # is deleted
-            atexit.register(delete, fn, allow_shred=False, ignore_missing=True)
-        if errors > 10:
-            break
-        if 'posix' == os.name and 0 == os.statvfs(pathname).f_ffree:
-            break
-
-    # flush once because it is slow
-    sync()
-
-    # display effectiveness
-    if 'posix' == os.name:
-        stats = os.statvfs(pathname)
-        logger.info('%d inodes available, %d to super user' %
-                    (stats.f_ffree, stats.f_favail))
-
-    # clean up
-    files_len_original = len(files)
-    logger.info('deleting %d empty files' % files_len_original)
-    for f in files:
-        try:
-            os.remove(f)
-        except:
-            if os.path.exists(f):
-                logger.warning('could not delete empty file %s' % f)
-            else:
-                logger.warning('exception deleting empty file %s' % f)
-        if idle and (time.time() - last_idle) > 2:
-            percent_done = 1.0 * \
-                (files_len_original - len(files)) / files_len_original
-            yield (3, percent_done, None)
-            last_idle = time.time()
-
-    os.rmdir(tmpdir)
-    logger.debug('done deleting empty files')
-
-
 def wipe_name(pathname1):
     """Wipe the original filename and return the new pathname"""
     logger = logging.getLogger(__name__)
@@ -673,7 +586,7 @@ def wipe_name(pathname1):
                 maxlen -= 10
             i += 1
             if i > 100:
-                logger.warning('exhausted long rename: %s' % pathname1)
+                logger.info('exhausted long rename: %s', pathname1)
                 pathname2 = pathname1
                 break
     # finally, rename to a short name
@@ -686,7 +599,7 @@ def wipe_name(pathname1):
         except:
             i = i + 1
             if i > 100:
-                logger.warning('exhausted short rename: %s' % pathname2)
+                logger.info('exhausted short rename: %s', pathname2)
                 pathname3 = pathname2
                 break
     return pathname3
@@ -744,7 +657,7 @@ def wipe_path(pathname, idle=False):
         remaining_seconds = int(remaining_bytes / (rate + 0.0001))
         return (1, done_percent, remaining_seconds)
 
-    logger.debug("wipe_path('%s')" % pathname)
+    logger.debug("wipe_path('%s')", pathname)
     files = []
     total_bytes = 0
     start_free_bytes = free_space(pathname)
@@ -803,15 +716,15 @@ def wipe_path(pathname, idle=False):
     # statistics
     elapsed_sec = time.time() - start_time
     rate_mbs = (total_bytes / (1000 * 1000)) / elapsed_sec
-    logger.info('wrote %d files and %d bytes in %d seconds at %.2f MB/s' %
-                (len(files), total_bytes, elapsed_sec, rate_mbs))
+    logger.info('wrote %d files and %d bytes in %d seconds at %.2f MB/s',
+                len(files), total_bytes, elapsed_sec, rate_mbs)
     # how much free space is left (should be near zero)
     if 'posix' == os.name:
         stats = os.statvfs(pathname)
-        logger.info('%d bytes and %d inodes available to non-super-user' %
-                    (stats.f_bsize * stats.f_bavail, stats.f_favail))
-        logger.info('%d bytes and %d inodes available to super-user' %
-                    (stats.f_bsize * stats.f_bfree, stats.f_ffree))
+        logger.info('%d bytes and %d inodes available to non-super-user',
+                    stats.f_bsize * stats.f_bavail, stats.f_favail)
+        logger.info('%d bytes and %d inodes available to super-user',
+                    stats.f_bsize * stats.f_bfree, stats.f_ffree)
     # truncate and close files
     for f in files:
         f.truncate(0)
