@@ -1,8 +1,8 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2015 Andrew Ziem
-# http://bleachbit.sourceforge.net
+# Copyright (C) 2008-2016 Andrew Ziem
+# http://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -41,12 +41,12 @@ def __shred_sqlite_char_columns(table, cols=None, where=""):
     """Create an SQL command to shred character columns"""
     cmd = ""
     if cols and options.get('shred'):
-        cmd += "update %s set %s %s;" % \
+        cmd += "update or ignore %s set %s %s;" % \
             (table, ",".join(["%s = randomblob(length(%s))" % (col, col)
-             for col in cols]), where)
-        cmd += "update %s set %s %s;" % \
+                              for col in cols]), where)
+        cmd += "update or ignore %s set %s %s;" % \
             (table, ",".join(["%s = zeroblob(length(%s))" % (col, col)
-             for col in cols]), where)
+                              for col in cols]), where)
     cmd += "delete from %s %s;" % (table, where)
     return cmd
 
@@ -97,7 +97,7 @@ def delete_chrome_autofill(path):
     cmds += __shred_sqlite_char_columns('autofill_profiles', cols)
     cols = (
         'company_name', 'street_address', 'address_1', 'address_2', 'address_3', 'address_4',
-            'postal_code', 'country_code', 'language_code', 'recipient_name', 'phone_number')
+        'postal_code', 'country_code', 'language_code', 'recipient_name', 'phone_number')
     cmds += __shred_sqlite_char_columns('server_addresses', cols)
     FileUtilities.execute_sqlite3(path, cmds)
 
@@ -117,7 +117,7 @@ def delete_chrome_favicons(path):
     ver = __get_chrome_history(path)
     cmds = ""
 
-    if ver in [4, 20, 22, 23, 25, 26, 28, 29]:
+    if ver in [4, 20, 22, 23, 25, 26, 28, 29, 32]:
         # Version 4 includes Chromium 12
         # Version 20 includes Chromium 14, Google Chrome 15, Google Chrome 19
         # Version 22 includes Google Chrome 20
@@ -125,6 +125,7 @@ def delete_chrome_favicons(path):
         # Version 26 is Google Chrome 29
         # Version 28 is Google Chrome 30
         # Version 29 is Google Chrome 37
+        # Version 32 is Google Chrome 51
 
         # icon_mapping
         cols = ('page_url',)
