@@ -2,8 +2,8 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2015 Andrew Ziem
-# http://bleachbit.sourceforge.net
+# Copyright (C) 2008-2016 Andrew Ziem
+# http://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@ Integration specific to Unix-like operating systems
 
 
 import glob
+import logging
 import os
 import re
 import shlex
@@ -36,126 +37,234 @@ import Common
 import FileUtilities
 import General
 
-HAVE_GNOME_VFS = True
-try:
-    import gnomevfs
-except:
-    try:
-        # this is the deprecated name
-        import gnome.vfs
-    except:
-        HAVE_GNOME_VFS = False
-    else:
-        gnomevfs = gnome.vfs
-
-
-def locale_to_language(locale):
-    """Convert the locale code to a language code (generally ISO 639)"""
-    if 'klingon' == locale:
-        return locale
-    pattern = "^([a-z]{2,3})([_-][a-zA-Z]{2,4})?(\.[a-zA-Z0-9-]*)?(@[a-zA-Z]*)?$"
-    matches = re.findall(pattern, locale)
-    if 1 > len(matches):
-        raise ValueError("Invalid locale_code '%s'" % (locale,))
-    return matches[0][0]
-
-
-def locale_globex(globpath, regex):
-    """List a path by glob, filter by regex, and return tuple
-    in format (locale, pathname)"""
-    for pathname in FileUtilities.globex(globpath, regex):
-        match = re.search(regex, pathname)
-        if None == match:
-            continue
-        locale_code = match.groups(0)[0]
-        yield (locale_code, pathname)
-
 
 class Locales:
 
     """Find languages and localization files"""
 
+    localepattern = r"(?P<locale>[a-z]{2,3})(?:[_-][a-zA-Z]{2,4})?(?:\.[a-zA-Z0-9-]*)?(?:@[a-zA-Z]*)?"
+
     native_locale_names = \
-        {'ar': 'العربية',
+        {'aa': 'Afaraf',
+         'ab': 'аҧсуа бызшәа',
+         'ach': 'Acoli',
+         'ae': 'avesta',
+         'af': 'Afrikaans',
+         'ak': 'Akan',
+         'am': 'አማርኛ',
+         'an': 'aragonés',
+         'ang': 'Old English',
+         'ar': 'العربية',
+         'as': 'অসমীয়া',
          'ast': 'Asturianu',
+         'av': 'авар мацӀ',
+         'ay': 'aymar aru',
+         'az': 'azərbaycan dili',
+         'ba': 'башҡорт теле',
+         'bal': 'Baluchi',
          'be': 'Беларуская мова',
          'bg': 'български език',
+         'bh': 'भोजपुरी',
+         'bi': 'Bislama',
+         'bm': 'bamanankan',
          'bn': 'বাংলা',
+         'bo': 'བོད་ཡིག',
+         'br': 'brezhoneg',
          'bs': 'босански',
+         'byn': 'Bilin',
          'ca': 'català',
+         'ce': 'нохчийн мотт',
+         'cgg': 'Chiga',
+         'ch': 'Chamoru',
+         'ckb': 'Central Kurdish',
+         'co': 'corsu',
+         'cr': 'ᓀᐦᐃᔭᐍᐏᐣ',
+         'crh': 'Crimean Tatar',
          'cs': 'česky',
+         'csb': 'Cashubian',
+         'cu': 'ѩзыкъ словѣньскъ',
+         'cv': 'чӑваш чӗлхи',
+         'cy': 'Cymraeg',
          'da': 'dansk',
          'de': 'Deutsch',
+         'dv': 'ދިވެހި',
+         'dz': 'རྫོང་ཁ',
+         'ee': 'Eʋegbe',
          'el': 'Ελληνικά',
          'en': 'English',
          'en_AU': 'Australian English',
+         'en_CA': 'Canadian English',
          'en_GB': 'British English',
          'eo': 'Esperanto',
          'es': 'Español',
          'et': 'eesti',
+         'eu': 'euskara',
          'fa': 'فارسی',
+         'ff': 'Fulfulde',
          'fi': 'suomen kieli',
+         'fj': 'vosa Vakaviti',
          'fo': 'føroyskt',
          'fr': 'Français',
+         'fur': 'Frilian',
+         'fy': 'Frysk',
+         'ga': 'Gaeilge',
+         'gd': 'Gàidhlig',
+         'gez': 'Geez',
          'gl': 'galego',
+         'gn': 'Avañeẽ',
+         'gu': 'Gujarati',
+         'gv': 'Gaelg',
+         'ha': 'هَوُسَ',
+         'haw': 'Hawaiian',
          'he': 'עברית',
          'hi': 'हिन्दी',
+         'hne': 'Chhattisgarhi',
+         'ho': 'Hiri Motu',
          'hr': 'Hrvatski',
+         'hsb': 'Upper Sorbian',
+         'ht': 'Kreyòl ayisyen',
          'hu': 'Magyar',
          'hy': 'Հայերեն',
+         'hz': 'Otjiherero',
          'ia': 'Interlingua',
          'id': 'Indonesian',
+         'ig': 'Asụsụ Igbo',
+         'ii': 'ꆈꌠ꒿',
+         'ik': 'Iñupiaq',
+         'io': 'Ido',
+         'is': 'Íslenska',
          'it': 'Italiano',
+         'iu': 'ᐃᓄᒃᑎᑐᑦ',
          'iw': 'עברית',
-         'ku': 'Kurdî',
-         'ky': 'Кыргызча',
          'ja': '日本語',
          'jv': 'basa Jawa',
-         'lt': 'lietuvių kalba',
+         'ka': 'ქართული',
+         'kg': 'Kikongo',
+         'ki': 'Gĩkũyũ',
+         'kj': 'Kuanyama',
+         'kk': 'қазақ тілі',
+         'kl': 'kalaallisut',
+         'km': 'ខ្មែរ',
+         'kn': 'ಕನ್ನಡ',
          'ko': '한국어',
+         'kok': 'Konkani',
+         'kr': 'Kanuri',
+         'ks': 'कश्मीरी',
+         'ku': 'Kurdî',
+         'kv': 'коми кыв',
+         'kw': 'Kernewek',
+         'ky': 'Кыргызча',
+         'la': 'latine',
+         'lb': 'Lëtzebuergesch',
+         'lg': 'Luganda',
+         'li': 'Limburgs',
+         'ln': 'Lingála',
+         'lo': 'ພາສາລາວ',
+         'lt': 'lietuvių kalba',
+         'lu': 'Tshiluba',
+         'lv': 'latviešu valoda',
+         'mai': 'Maithili',
+         'mg': 'fiteny malagasy',
+         'mh': 'Kajin M̧ajeļ',
+         'mhr': 'Eastern Mari',
+         'mi': 'te reo Māori',
+         'mk': 'македонски јазик',
+         'ml': 'മലയാളം',
+         'mn': 'монгол',
          'mr': 'मराठी',
          'ms': 'بهاس ملايو',
+         'mt': 'Malti',
          'my': 'ဗမာစာ',
+         'na': 'Ekakairũ Naoero',
          'nb': 'Bokmål',
+         'nd': 'isiNdebele',
          'nds': 'Plattdüütsch',
+         'ne': 'नेपाली',
+         'ng': 'Owambo',
          'nl': 'Nederlands',
+         'nn': 'Norsk nynorsk',
          'no': 'Norsk',
+         'nr': 'isiNdebele',
+         'nso': 'Pedi',
+         'nv': 'Diné bizaad',
+         'ny': 'chiCheŵa',
+         'oc': 'occitan',
+         'oj': 'ᐊᓂᔑᓈᐯᒧᐎᓐ',
+         'om': 'Afaan Oromoo',
+         'or': 'ଓଡ଼ିଆ',
+         'os': 'ирон æвзаг',
+         'pa': 'ਪੰਜਾਬੀ',
+         'pi': 'पाऴि',
          'pl': 'polski',
+         'ps': 'پښتو',
          'pt': 'Português',
+         'pt_BR': 'Português do Brasil',
+         'qu': 'Runa Simi',
+         'rm': 'rumantsch grischun',
+         'rn': 'Ikirundi',
          'ro': 'română',
          'ru': 'Pусский',
+         'rw': 'Ikinyarwanda',
+         'sa': 'संस्कृतम्',
+         'sc': 'sardu',
+         'sd': 'सिन्धी',
+         'se': 'Davvisámegiella',
+         'sg': 'yângâ tî sängö',
+         'shn': 'Shan',
+         'si': 'සිංහල',
          'sk': 'slovenčina',
          'sl': 'slovenščina',
+         'sm': 'gagana faa Samoa',
+         'sn': 'chiShona',
+         'so': 'Soomaaliga',
+         'sq': 'Shqip',
          'sr': 'Српски',
+         'ss': 'SiSwati',
+         'st': 'Sesotho',
+         'su': 'Basa Sunda',
          'sv': 'svenska',
+         'sw': 'Kiswahili',
+         'ta': 'தமிழ்',
+         'te': 'తెలుగు',
+         'tet': 'Tetum',
+         'tg': 'тоҷикӣ',
+         'th': 'ไทย',
+         'ti': 'ትግርኛ',
+         'tig': 'Tigre',
+         'tk': 'Türkmen',
+         'tl': 'ᜏᜒᜃᜅ᜔ ᜆᜄᜎᜓᜄ᜔',
+         'tn': 'Setswana',
+         'to': 'faka Tonga',
          'tr': 'Türkçe',
+         'ts': 'Xitsonga',
+         'tt': 'татар теле',
+         'tw': 'Twi',
+         'ty': 'Reo Tahiti',
          'ug': 'Uyghur',
          'uk': 'Українська',
+         'ur': 'اردو',
+         'uz': 'Ўзбек',
+         've': 'Tshivenḓa',
          'vi': 'Tiếng Việt',
-         'zh': '中文'}
+         'vo': 'Volapük',
+         'wa': 'walon',
+         'wae': 'Walser',
+         'wal': 'Wolaytta',
+         'wo': 'Wollof',
+         'xh': 'isiXhosa',
+         'yi': 'ייִדיש',
+         'yo': 'Yorùbá',
+         'za': 'Saɯ cueŋƅ',
+         'zh': '中文',
+         'zh_CN': '中文',
+         'zh_TW': '中文',
+         'zu': 'isiZulu'}
 
     _paths = []
 
     def __init__(self):
         pass
 
-    def native_name(self, language_code):
-        """Return the name of the language in its own language"""
-        if language_code in self.native_locale_names:
-            return self.native_locale_names[language_code]
-        if os.path.exists('/usr/share/locale/all_languages'):
-            if not self.__config_read:
-                # In Fedora 15, this file is provided by kdelibs-common
-                self.__config.read('/usr/share/locale/all_languages')
-                self.__config_read = True
-            option = 'Name[%s]' % (language_code, )
-            if self.__config.has_option(language_code, option):
-                value = self.__config.get(language_code, option)
-                # cache
-                self.native_locale_names[language_code] = value
-                return value
-        return None
-
     def add_xml(self, xml_node):
         self._paths.append(xml_node)
 
@@ -163,7 +272,7 @@ class Locales:
         if not locales_to_keep:
             raise RuntimeError('Found no locales to keep')
         purgeable_locales = frozenset((locale for locale in Locales.native_locale_names.keys()
-                                      if locale not in locales_to_keep))
+                                       if locale not in locales_to_keep))
 
         for xml_node in self._paths:
             for (locale, path) in Locales.handle_path('', xml_node):
@@ -172,46 +281,55 @@ class Locales:
 
     @staticmethod
     def handle_path(path, xmldata):
+        """Extracts paths and filters from the supplied xml tree and yields matching files"""
 
         if xmldata.ELEMENT_NODE != xmldata.nodeType:
             return
-        if 'path' != xmldata.nodeName:
+        if not xmldata.nodeName in ['path', 'regexfilter']:
             raise RuntimeError(
-                "Invalid node '%s', expected 'path'" % xmldata.nodeName)
-        location = xmldata.getAttribute('location')
-        if '' == location:
-            raise RuntimeError("Path node without location attribute")
-        path = path + location
+                "Invalid node '%s', expected '<path>' or '<regexfilter>'" % xmldata.nodeName)
+        location = xmldata.getAttribute('location') or '.'
+        if '.' != location:
+            path = path + location
         if not path.endswith('/'):
             path += '/'
 
         if not os.path.isdir(path):
             return
 
-        userfilter = xmldata.getAttribute('filter')
-        if not userfilter and not xmldata.hasChildNodes():
-            userfilter = '*'
+        pre = None
+        post = None
+        if 'regexfilter' == xmldata.nodeName:
+            pre = xmldata.getAttribute('prefix') or ''
+            post = xmldata.getAttribute('postfix') or ''
+        if 'path' == xmldata.nodeName:
+            userfilter = xmldata.getAttribute('filter')
+            if not userfilter and not xmldata.hasChildNodes():
+                userfilter = '*'
+            if userfilter:
+                if 1 != userfilter.count('*'):
+                    raise RuntimeError(
+                        "Filter string '%s' must contain the placeholder * exactly once" % userfilter)
 
-        if userfilter:
-            if 1 != userfilter.count('*'):
+                # we can't use re.escape, because it escapes too much
+                (pre, post) = (re.sub(r'([\[\]()^$.])', r'\\\1', p)
+                               for p in userfilter.split('*'))
+            # handle child nodes
+            for child in xmldata.childNodes:
+                for (locale, subpath) in Locales.handle_path(path, child):
+                    yield (locale, subpath)
+        if pre is not None and post is not None:
+            try:
+                re.compile(pre)
+                re.compile(post)
+            except Exception as errormsg:
                 raise RuntimeError(
-                    "Filter string '%s' must contain the placeholder * exactly once" % userfilter)
-            (prefixlen, postfixlen) = [len(part)
-                                       for part in userfilter.split('*')]
-            postfixlen = -postfixlen if 0 != postfixlen else None
-
-            for subpath in glob.iglob(path + userfilter):
-                try:
-                    filename = os.path.split(subpath)[1]
-                    locale = locale_to_language(filename[prefixlen:postfixlen])
-                except ValueError:
-                    continue
-                yield (locale, subpath)
-
-        # handle child nodes
-        for child in xmldata.childNodes:
-            for (locale, subpath) in Locales.handle_path(path, child):
-                yield (locale, subpath)
+                    "Malformed regex '%s' or '%s': %s" % (pre, post, errormsg))
+            regex = re.compile('^' + pre + Locales.localepattern + post + '$')
+            for subpath in os.listdir(path):
+                match = regex.match(subpath)
+                if match is not None:
+                    yield (match.group("locale"), path + subpath)
 
 
 def apt_autoclean():
@@ -264,7 +382,7 @@ def apt_autoremove():
             raise RuntimeError(line)
         # After this operation, 74.7MB disk space will be freed.
         match = re.search(
-            ", ([0-9.]+[a-zA-Z]{2}) disk space will be freed", line)
+            r", ([0-9.]+[a-zA-Z]{2}) disk space will be freed", line)
         if match:
             pkg_bytes_str = match.groups(0)[0]
             pkg_bytes = FileUtilities.human_to_bytes(pkg_bytes_str.upper())
@@ -292,13 +410,13 @@ def __is_broken_xdg_desktop_application(config, desktop_pathname):
         # Files\\foo\\foo.exe"
         execs = shlex.split(config.get('Desktop Entry', 'Exec'))
         wineprefix = None
-        del(execs[0])
+        del execs[0]
         while True:
             if 0 <= execs[0].find("="):
                 (name, value) = execs[0].split("=")
                 if 'WINEPREFIX' == name:
                     wineprefix = value
-                del(execs[0])
+                del execs[0]
             else:
                 break
         if not FileUtilities.exe_exists(execs[0]):
@@ -316,6 +434,20 @@ def __is_broken_xdg_desktop_application(config, desktop_pathname):
     return False
 
 
+def is_unregistered_mime(mimetype):
+    """Returns True if the MIME type is known to be unregistered. If
+    registered or unknown, conservatively returns False."""
+    try:
+        import gio
+        if 0 == len(gio.app_info_get_all_for_type(mimetype)):
+            return True
+    except:
+        logger = logging.getLogger(__name__)
+        logger.warning(
+            'error calling gio.app_info_get_all_for_type(%s)' % mimetype)
+    return False
+
+
 def is_broken_xdg_desktop(pathname):
     """Returns boolean whether the given XDG desktop entry file is broken.
     Reference: http://standards.freedesktop.org/desktop-entry-spec/latest/"""
@@ -340,7 +472,7 @@ def is_broken_xdg_desktop(pathname):
             print "info: is_broken_xdg_menu: missing required option 'MimeType': '%s'" % (pathname)
             return True
         mimetype = config.get('Desktop Entry', 'MimeType').strip().lower()
-        if HAVE_GNOME_VFS and 0 == len(gnomevfs.mime_get_all_applications(mimetype)):
+        if is_unregistered_mime(mimetype):
             print "info: is_broken_xdg_menu: MimeType '%s' not " \
                 "registered '%s'" % (mimetype, pathname)
             return True
