@@ -1,8 +1,9 @@
+
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2015 Andrew Ziem
-# http://bleachbit.sourceforge.net
+# Copyright (C) 2008-2016 Andrew Ziem
+# http://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -34,7 +35,6 @@ import traceback
 
 from Action import Delete, Winreg
 from Common import _
-from platform import uname
 from xml.dom.minidom import parseString
 
 
@@ -80,19 +80,22 @@ def section2option(s):
 def detectos(required_ver, mock=False):
     """Returns boolean whether the detectos is compatible with the
     current operating system, or the mock version, if given."""
-    current_os = (mock if mock else uname()[3])[0:3]
+    # Do not compare as string because Windows 10 (build 10.0) comes after
+    # Windows 8.1 (build 6.3).
+    assert(isinstance(required_ver, (str, unicode)))
+    current_os = (mock if mock else Windows.parse_windows_build())
     required_ver = required_ver.strip()
     if required_ver.startswith('|'):
         # This is the maximum version
         # For example, |5.1 means Windows XP (5.1) but not Vista (6.0)
-        return current_os <= required_ver[1:]
+        return current_os <= Windows.parse_windows_build(required_ver[1:])
     elif required_ver.endswith('|'):
         # This is the minimum version
         # For example, 6.1| means Windows 7 or later
-        return current_os >= required_ver[:-1]
+        return current_os >= Windows.parse_windows_build(required_ver[:-1])
     else:
         # Exact version
-        return required_ver == current_os
+        return Windows.parse_windows_build(required_ver) == current_os
 
 
 def winapp_expand_vars(pathname):
@@ -120,6 +123,17 @@ def detect_file(pathname):
     return False
 
 
+def fnmatch_translate(pattern):
+    """Same as the original without the end"""
+    import fnmatch
+    r = fnmatch.translate(pattern)
+    if r.endswith('$'):
+        return r[:-1]
+    if r.endswith(r'\Z(?ms)'):
+        return r[:-7]
+    return r
+
+
 class Winapp:
 
     """Create cleaners from a Winapp2.ini-style file"""
@@ -163,6 +177,57 @@ class Winapp:
             self.add_section(cleanerid, langsecref)
         return cleanerid
 
+    def excludekey_to_nwholeregex(self, excludekey):
+        """Translate one ExcludeKey to CleanerML nwholeregex
+
+        Supported examples
+        FILE=%LocalAppData%\BleachBit\BleachBit.ini
+        FILE=%LocalAppData%\BleachBit\|BleachBit.ini
+        FILE=%LocalAppData%\BleachBit\|*.ini
+        FILE=%LocalAppData%\BleachBit\|*.ini;*.bak
+        PATH=%LocalAppData%\BleachBit\
+        PATH=%LocalAppData%\BleachBit\|*.*
+        """
+        parts = excludekey.split('|')
+        parts[0] = parts[0].upper()
+        if 'REG' == parts[0]:
+            raise NotImplementedError('REG not supported in ExcludeKey')
+
+        # the last part contains the filename(s)
+        files = None
+        files_regex = ''
+        if 3 == len(parts):
+            files = parts[2].split(';')
+            if 1 == len(files):
+                # one file pattern like *.* or *.log
+                files_regex = fnmatch_translate(files[0])
+                if '*.*' == files_regex:
+                    files = None
+            elif len(files) > 1:
+                # multiple file patterns like *.log;*.bak
+                files_regex = '(%s)' % '|'.join(
+                    [fnmatch_translate(f) for f in files])
+
+        # the middle part contains the file
+        regexes = []
+        for expanded in winapp_expand_vars(parts[1]):
+            regex = None
+            if not files:
+                # There is no third part, so this is either just a folder,
+                # or sometimes the file is specified directly.
+                regex = fnmatch_translate(expanded)
+            if files:
+                # match one or more file types, directly in this tree or in any
+                # sub folder
+                regex = '%s.*%s' % (
+                    fnmatch_translate(expanded), files_regex)
+            regexes.append(regex)
+
+        if 1 == len(regexes):
+            return regexes[0]
+        else:
+            return '(%s)' % '|'.join(regexes)
+
     def handle_section(self, section):
         """Parse a section"""
         # if simple detection fails then discard the section
@@ -196,10 +261,14 @@ class Winapp:
                         matches = matches + 1
             if 0 == matches:
                 return
-        # not yet implemented
-        if self.parser.has_option(section, 'excludekey'):
-            print 'ERROR: ExcludeKey not implemented, section=', section
-            return
+        # excludekeys ignores a file, path, or registry key
+        excludekeys = []
+        if self.parser.has_option(section, 'excludekey1'):
+            for n in range(1, MAX_DETECT):
+                option_id = 'excludekey%d' % n
+                if self.parser.has_option(section, option_id):
+                    excludekeys.append(
+                        self.excludekey_to_nwholeregex(self.parser.get(section, option_id)))
         # there are two ways to specify sections: langsecref= and section=
         if self.parser.has_option(section, 'langsecref'):
             # verify the langsecref number is known
@@ -216,7 +285,7 @@ class Winapp:
             section2option(section), section.replace('*', ''), '')
         for option in self.parser.options(section):
             if option.startswith('filekey'):
-                self.handle_filekey(lid, section, option)
+                self.handle_filekey(lid, section, option, excludekeys)
             elif option.startswith('regkey'):
                 self.handle_regkey(lid, section, option)
             elif option == 'warning':
@@ -230,7 +299,7 @@ class Winapp:
                 print 'WARNING: unknown option %s in section %s' % (option, section)
                 return
 
-    def __make_file_provider(self, dirname, filename, recurse, removeself):
+    def __make_file_provider(self, dirname, filename, recurse, removeself, excludekeys):
         """Change parsed FileKey to action provider"""
         regex = ''
         if recurse:
@@ -242,21 +311,31 @@ class Winapp:
                 if removeself:
                     search = 'walk.all'
             else:
-                regex = ' regex="%s" ' % (re.escape(filename) + '$')
+                import fnmatch
+                regex = ' regex="%s" ' % (fnmatch.translate(filename))
         else:
             search = 'glob'
             path = os.path.join(dirname, filename)
             if -1 == path.find('*'):
                 search = 'file'
-        action_str = '<option command="delete" search="%s" path="%s" %s/>' % \
-            (search, xml_escape(path), regex)
+        excludekeysxml = ''
+        if excludekeys:
+            if len(excludekeys) > 1:
+                # multiple
+                exclude_str = '(%s)' % '|'.join(excludekeys)
+            else:
+                # just one
+                exclude_str = excludekeys[0]
+            excludekeysxml = 'nwholeregex="%s"' % exclude_str
+        action_str = '<option command="delete" search="%s" path="%s" %s %s/>' % \
+            (search, xml_escape(path), regex, excludekeysxml)
         yield Delete(parseString(action_str).childNodes[0])
         if removeself:
             action_str = '<option command="delete" search="file" path="%s"/>' % xml_escape(
                 dirname)
             yield Delete(parseString(action_str).childNodes[0])
 
-    def handle_filekey(self, lid, ini_section, ini_option):
+    def handle_filekey(self, lid, ini_section, ini_option, excludekeys):
         """Parse a FileKey# option.
 
         Section is [Application Name] and option is the FileKey#"""
@@ -278,7 +357,7 @@ class Winapp:
                 print 'WARNING: unknown file option %s in section %s' % (element, ini_section)
         for filename in filenames.split(';'):
             for dirname in dirnames:
-                for provider in self.__make_file_provider(dirname, filename, recurse, removeself):
+                for provider in self.__make_file_provider(dirname, filename, recurse, removeself, excludekeys):
                     self.cleaners[lid].add_action(
                         section2option(ini_section), provider)
 
